/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.seatunnel.transform.condition;

import lombok.Getter;
import lombok.Setter;
import org.apache.seatunnel.api.configuration.Option;
import org.apache.seatunnel.api.configuration.Options;
import org.apache.seatunnel.api.configuration.ReadonlyConfig;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

@Getter
@Setter
public class ConditionTransformConfig implements Serializable {
    public static final Option<List<ConditionEntity>> FIELD_CONDITION =
            Options.key("field_condition")
                    .listType(ConditionEntity.class)
                    .noDefaultValue()
                    .withDescription(
                            "Specify the field filtering criteria");
    public static final Option<String> CONNECT_TYPE =
            Options.key("connect_type")
                    .stringType()
                    .noDefaultValue()
                    .withDescription("Specify the field connect type");

    private List<ConditionEntity> fieldCondition = new ArrayList<>();

    private String connectType;


    public static ConditionTransformConfig of(ReadonlyConfig config) {
        ConditionTransformConfig conditionTransformConfig = new ConditionTransformConfig();
        conditionTransformConfig.setFieldCondition(config.get(FIELD_CONDITION));
        conditionTransformConfig.setConnectType(config.get(CONNECT_TYPE));
        return conditionTransformConfig;
    }
}
